@@ -15,25 +15,44 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #include "ooutils.h"
-#include "stylestack.h"
+#include <KoDocument.h>
+#include <KoStyleStack.h>
 #include <qdom.h>
 #include <qcolor.h>
-#include <koUnit.h>
+#include <qimage.h>
+#include <KoUnit.h>
 #include <qregexp.h>
 #include <kdebug.h>
+#include <kzip.h>
+#include <KoDom.h>
+#include <qxml.h>
+
+const char* const ooNS::office="http://openoffice.org/2000/office";
+const char* const ooNS::style="http://openoffice.org/2000/style";
+const char* const ooNS::text="http://openoffice.org/2000/text";
+const char* const ooNS::table="http://openoffice.org/2000/table";
+const char* const ooNS::draw="http://openoffice.org/2000/drawing";
+const char* const ooNS::presentation="http://openoffice.org/2000/presentation";
+const char* const ooNS::fo="http://www.w3.org/1999/XSL/Format";
+const char* const ooNS::xlink="http://www.w3.org/1999/xlink";
+const char* const ooNS::number="http://openoffice.org/2000/datastyle";
+const char* const ooNS::svg="http://www.w3.org/2000/svg";
+const char* const ooNS::dc="http://purl.org/dc/elements/1.1/";
+const char* const ooNS::meta="http://openoffice.org/2000/meta";
+const char* const ooNS::config="http://openoffice.org/2001/config";
 
 QString OoUtils::expandWhitespace(const QDomElement& tag)
 {
     //tags like <text:s text:c="4">
 
     int howmany=1;
-    if (tag.hasAttribute("text:c"))
-        howmany = tag.attribute("text:c").toInt();
+    if (tag.hasAttributeNS( ooNS::text, "c"))
+        howmany = tag.attributeNS( ooNS::text, "c", QString::null).toInt();
 
     QString result;
     return result.fill(32, howmany);
@@ -73,22 +92,22 @@ bool OoUtils::parseBorder(const QString & tag, double * width, int * style, QCol
     return true;
 }
 
-void OoUtils::importIndents( QDomElement& parentElement, const StyleStack& styleStack )
+void OoUtils::importIndents( QDomElement& parentElement, const KoStyleStack& styleStack )
 {
-    if ( styleStack.hasAttribute( "fo:margin-left" ) || // 3.11.19
-         styleStack.hasAttribute( "fo:margin-right" ) )
+    if ( styleStack.hasAttributeNS( ooNS::fo, "margin-left" ) || // 3.11.19
+         styleStack.hasAttributeNS( ooNS::fo, "margin-right" ) )
          // *text-indent must always be bound to either margin-left or margin-right
     {
-        double marginLeft = KoUnit::parseValue( styleStack.attribute( "fo:margin-left" ) );
-        double marginRight = KoUnit::parseValue( styleStack.attribute( "fo:margin-right" ) );
+        double marginLeft = KoUnit::parseValue( styleStack.attributeNS( ooNS::fo, "margin-left" ) );
+        double marginRight = KoUnit::parseValue( styleStack.attributeNS( ooNS::fo, "margin-right" ) );
         double first = 0;
-        if (styleStack.attribute("style:auto-text-indent") == "true") // style:auto-text-indent takes precedence
+        if (styleStack.attributeNS( ooNS::style, "auto-text-indent") == "true") // style:auto-text-indent takes precedence
             // ### "indented by a value that is based on the current font size"
             // ### and "requires margin-left and margin-right
             // ### but how much is the indent?
             first = 10;
-        else if (styleStack.hasAttribute("fo:text-indent"))
-            first = KoUnit::parseValue( styleStack.attribute("fo:text-indent"));
+        else if (styleStack.hasAttributeNS( ooNS::fo, "text-indent"))
+            first = KoUnit::parseValue( styleStack.attributeNS( ooNS::fo, "text-indent"));
 
         if ( marginLeft != 0 || marginRight != 0 || first != 0 )
         {
@@ -104,12 +123,12 @@ void OoUtils::importIndents( QDomElement& parentElement, const StyleStack& style
     }
 }
 
-void OoUtils::importLineSpacing( QDomElement& parentElement, const StyleStack& styleStack )
+void OoUtils::importLineSpacing( QDomElement& parentElement, const KoStyleStack& styleStack )
 {
-    if( styleStack.hasAttribute("fo:line-height") )
+    if( styleStack.hasAttributeNS( ooNS::fo, "line-height") )
     {
         // Fixed line height
-        QString value = styleStack.attribute( "fo:line-height" ); // 3.11.1
+        QString value = styleStack.attributeNS( ooNS::fo, "line-height" ); // 3.11.1
         if ( value != "normal" )
         {
             QDomElement lineSpacing = parentElement.ownerDocument().createElement( "LINESPACING" );
@@ -127,19 +146,19 @@ void OoUtils::importLineSpacing( QDomElement& parentElement, const StyleStack& s
             }
             else // fixed value (use KoUnit::parseValue to get it in pt)
             {
-                kdWarning() << "Unhandled value for fo:line-height: " << value << endl;
+                kdWarning(30519) << "Unhandled value for fo:line-height: " << value << endl;
             }
             parentElement.appendChild( lineSpacing );
         }
     }
     // Line-height-at-least is mutually exclusive with line-height
-    else if ( styleStack.hasAttribute("style:line-height-at-least") ) // 3.11.2
+    else if ( styleStack.hasAttributeNS( ooNS::style, "line-height-at-least") ) // 3.11.2
     {
-        QString value = styleStack.attribute( "style:line-height-at-least" );
+        QString value = styleStack.attributeNS( ooNS::style, "line-height-at-least" );
         // kotext has "at least" but that's for the linespacing, not for the entire line height!
         // Strange. kotext also has "at least" for the whole line height....
         // Did we make the wrong choice in kotext?
-        //kdWarning() << "Unimplemented support for style:line-height-at-least: " << value << endl;
+        //kdWarning(30519) << "Unimplemented support for style:line-height-at-least: " << value << endl;
         // Well let's see if this makes a big difference.
         QDomElement lineSpacing = parentElement.ownerDocument().createElement("LINESPACING");
         lineSpacing.setAttribute("type", "atleast");
@@ -147,9 +166,9 @@ void OoUtils::importLineSpacing( QDomElement& parentElement, const StyleStack& s
         parentElement.appendChild(lineSpacing);
     }
     // Line-spacing is mutually exclusive with line-height and line-height-at-least
-    else if ( styleStack.hasAttribute("style:line-spacing") ) // 3.11.3
+    else if ( styleStack.hasAttributeNS( ooNS::style, "line-spacing") ) // 3.11.3
     {
-        double value = KoUnit::parseValue( styleStack.attribute( "style:line-spacing" ) );
+        double value = KoUnit::parseValue( styleStack.attributeNS( ooNS::style, "line-spacing" ) );
         if ( value != 0.0 )
         {
             QDomElement lineSpacing = parentElement.ownerDocument().createElement( "LINESPACING" );
@@ -161,13 +180,13 @@ void OoUtils::importLineSpacing( QDomElement& parentElement, const StyleStack& s
 
 }
 
-void OoUtils::importTopBottomMargin( QDomElement& parentElement, const StyleStack& styleStack )
+void OoUtils::importTopBottomMargin( QDomElement& parentElement, const KoStyleStack& styleStack )
 {
-    if( styleStack.hasAttribute("fo:margin-top") || // 3.11.22
-        styleStack.hasAttribute("fo:margin-bottom"))
+    if( styleStack.hasAttributeNS( ooNS::fo, "margin-top") || // 3.11.22
+        styleStack.hasAttributeNS( ooNS::fo, "margin-bottom"))
     {
-        double mtop = KoUnit::parseValue( styleStack.attribute( "fo:margin-top" ) );
-        double mbottom = KoUnit::parseValue( styleStack.attribute("fo:margin-bottom" ) );
+        double mtop = KoUnit::parseValue( styleStack.attributeNS( ooNS::fo, "margin-top" ) );
+        double mbottom = KoUnit::parseValue( styleStack.attributeNS( ooNS::fo, "margin-bottom" ) );
         if( mtop != 0 || mbottom != 0 )
         {
             QDomElement offset = parentElement.ownerDocument().createElement( "OFFSETS" );
@@ -180,17 +199,17 @@ void OoUtils::importTopBottomMargin( QDomElement& parentElement, const StyleStac
     }
 }
 
-void OoUtils::importTabulators( QDomElement& parentElement, const StyleStack& styleStack )
+void OoUtils::importTabulators( QDomElement& parentElement, const KoStyleStack& styleStack )
 {
-    if ( !styleStack.hasChildNode( "style:tab-stops" ) ) // 3.11.10
+    if ( !styleStack.hasChildNodeNS( ooNS::style, "tab-stops" ) ) // 3.11.10
         return;
-    QDomElement tabStops = styleStack.childNode( "style:tab-stops" ).toElement();
+    QDomElement tabStops = styleStack.childNodeNS( ooNS::style, "tab-stops" );
     //kdDebug(30519) << k_funcinfo << tabStops.childNodes().count() << " tab stops in layout." << endl;
     for ( QDomNode it = tabStops.firstChild(); !it.isNull(); it = it.nextSibling() )
     {
         QDomElement tabStop = it.toElement();
         Q_ASSERT( tabStop.tagName() == "style:tab-stop" );
-        QString type = tabStop.attribute( "style:type" ); // left, right, center or char
+        QString type = tabStop.attributeNS( ooNS::style, "type", QString::null ); // left, right, center or char
 
         QDomElement elem = parentElement.ownerDocument().createElement( "TABULATOR" );
         int kOfficeType = 0;
@@ -201,19 +220,19 @@ void OoUtils::importTabulators( QDomElement& parentElement, const StyleStack& st
         else if ( type == "right" )
             kOfficeType = 2;
         else if ( type == "char" ) {
-            QString delimiterChar = tabStop.attribute( "style:char" ); // single character
+            QString delimiterChar = tabStop.attributeNS( ooNS::style, "char", QString::null ); // single character
             elem.setAttribute( "alignchar", delimiterChar );
             kOfficeType = 3; // "alignment on decimal point"
         }
 
         elem.setAttribute( "type", kOfficeType );
 
-        double pos = KoUnit::parseValue( tabStop.attribute( "style:position" ) );
+        double pos = KoUnit::parseValue( tabStop.attributeNS( ooNS::style, "position", QString::null ) );
         elem.setAttribute( "ptpos", pos );
 
         // TODO Convert leaderChar's unicode value to the KOffice enum
         // (blank/dots/line/dash/dash-dot/dash-dot-dot, 0 to 5)
-        QString leaderChar = tabStop.attribute( "style:leader-char" ); // single character
+        QString leaderChar = tabStop.attributeNS( ooNS::style, "leader-char", QString::null ); // single character
         if ( !leaderChar.isEmpty() )
         {
             int filling = 0;
@@ -236,14 +255,14 @@ void OoUtils::importTabulators( QDomElement& parentElement, const StyleStack& st
 
 }
 
-void OoUtils::importBorders( QDomElement& parentElement, const StyleStack& styleStack )
+void OoUtils::importBorders( QDomElement& parentElement, const KoStyleStack& styleStack )
 {
-    if (styleStack.hasAttribute("fo:border","left"))
+    if (styleStack.hasAttributeNS( ooNS::fo, "border","left"))
     {
         double width;
         int style;
         QColor color;
-        if (OoUtils::parseBorder(styleStack.attribute("fo:border", "left"), &width, &style, &color))
+        if (OoUtils::parseBorder(styleStack.attributeNS( ooNS::fo, "border", "left"), &width, &style, &color))
         {
             QDomElement lbElem = parentElement.ownerDocument().createElement("LEFTBORDER");
             lbElem.setAttribute("width", width);
@@ -257,12 +276,12 @@ void OoUtils::importBorders( QDomElement& parentElement, const StyleStack& style
         }
     }
 
-    if (styleStack.hasAttribute("fo:border", "right"))
+    if (styleStack.hasAttributeNS( ooNS::fo, "border", "right"))
     {
         double width;
         int style;
         QColor color;
-        if (OoUtils::parseBorder(styleStack.attribute("fo:border", "right"), &width, &style, &color))
+        if (OoUtils::parseBorder(styleStack.attributeNS( ooNS::fo, "border", "right"), &width, &style, &color))
         {
             QDomElement lbElem = parentElement.ownerDocument().createElement("RIGHTBORDER");
             lbElem.setAttribute("width", width);
@@ -276,12 +295,12 @@ void OoUtils::importBorders( QDomElement& parentElement, const StyleStack& style
         }
     }
 
-    if (styleStack.hasAttribute("fo:border", "top"))
+    if (styleStack.hasAttributeNS( ooNS::fo, "border", "top"))
     {
         double width;
         int style;
         QColor color;
-        if (OoUtils::parseBorder(styleStack.attribute("fo:border", "top"), &width, &style, &color))
+        if (OoUtils::parseBorder(styleStack.attributeNS( ooNS::fo, "border", "top"), &width, &style, &color))
         {
             QDomElement lbElem = parentElement.ownerDocument().createElement("TOPBORDER");
             lbElem.setAttribute("width", width);
@@ -295,12 +314,12 @@ void OoUtils::importBorders( QDomElement& parentElement, const StyleStack& style
         }
     }
 
-    if (styleStack.hasAttribute("fo:border", "bottom"))
+    if (styleStack.hasAttributeNS( ooNS::fo, "border", "bottom"))
     {
         double width;
         int style;
         QColor color;
-        if (OoUtils::parseBorder(styleStack.attribute("fo:border", "bottom"), &width, &style, &color))
+        if (OoUtils::parseBorder(styleStack.attributeNS( ooNS::fo, "border", "bottom"), &width, &style, &color))
         {
             QDomElement lbElem = parentElement.ownerDocument().createElement("BOTTOMBORDER");
             lbElem.setAttribute("width", width);
@@ -315,7 +334,6 @@ void OoUtils::importBorders( QDomElement& parentElement, const StyleStack& style
     }
 }
 
-// TODO use this in OoImpressImport!
 void OoUtils::importUnderline( const QString& in, QString& underline, QString& styleline )
 {
     underline = "single";
@@ -357,7 +375,7 @@ void OoUtils::importUnderline( const QString& in, QString& underline, QString& s
         styleline = "solid";
     }
     else
-        kdWarning() << k_funcinfo << " unsupported text-underline value: " << in << endl;
+        kdWarning(30519) << k_funcinfo << " unsupported text-underline value: " << in << endl;
 }
 
 void OoUtils::importTextPosition( const QString& text_position, QString& value, QString& relativetextsize )
@@ -373,7 +391,8 @@ void OoUtils::importTextPosition( const QString& text_position, QString& value,
         lst.pop_front();
         if ( !lst.isEmpty() )
             textSize = lst.front().stripWhiteSpace();
-        Q_ASSERT( lst.isEmpty() );
+        if ( !lst.isEmpty() )
+            kdWarning(30519) << "Strange text position: " << text_position << endl;
         bool super = textPos == "super";
         bool sub = textPos == "sub";
         if ( textPos.endsWith("%") )
@@ -403,3 +422,201 @@ void OoUtils::importTextPosition( const QString& text_position, QString& value,
     else
         value = "0";
 }
+
+void OoUtils::createDocumentInfo(QDomDocument &_meta, QDomDocument & docinfo)
+{
+    QDomNode meta   = KoDom::namedItemNS( _meta, ooNS::office, "document-meta" );
+    QDomNode office = KoDom::namedItemNS( meta, ooNS::office, "meta" );
+
+    if ( office.isNull() )
+        return;
+    QDomElement elementDocInfo  = docinfo.documentElement();
+
+    QDomElement e = KoDom::namedItemNS( office, ooNS::dc, "creator" );
+    if ( !e.isNull() && !e.text().isEmpty() )
+    {
+        QDomElement author = docinfo.createElement( "author" );
+        QDomElement t = docinfo.createElement( "full-name" );
+        author.appendChild( t );
+        t.appendChild( docinfo.createTextNode( e.text() ) );
+        elementDocInfo.appendChild( author);
+    }
+
+    e = KoDom::namedItemNS( office, ooNS::dc, "title" );
+    if ( !e.isNull() && !e.text().isEmpty() )
+    {
+        QDomElement about = docinfo.createElement( "about" );
+        QDomElement title = docinfo.createElement( "title" );
+        about.appendChild( title );
+        title.appendChild( docinfo.createTextNode( e.text() ) );
+        elementDocInfo.appendChild( about );
+    }
+
+    e = KoDom::namedItemNS( office, ooNS::dc, "description" );
+    if ( !e.isNull() && !e.text().isEmpty() )
+    {
+        QDomElement about = elementDocInfo.namedItem( "about" ).toElement();
+        if ( about.isNull() ) {
+            about = docinfo.createElement( "about" );
+            elementDocInfo.appendChild( about );
+        }
+        QDomElement title = docinfo.createElement( "abstract" );
+        about.appendChild( title );
+        title.appendChild( docinfo.createTextNode( e.text() ) );
+    }
+    e = KoDom::namedItemNS( office, ooNS::dc, "subject" );
+    if ( !e.isNull() && !e.text().isEmpty() )
+    {
+        QDomElement about = elementDocInfo.namedItem( "about" ).toElement();
+        if ( about.isNull() ) {
+            about = docinfo.createElement( "about" );
+            elementDocInfo.appendChild( about );
+        }
+        QDomElement subject = docinfo.createElement( "subject" );
+        about.appendChild( subject );
+        subject.appendChild( docinfo.createTextNode( e.text() ) );
+    }
+    e = KoDom::namedItemNS( office, ooNS::meta, "keywords" );
+    if ( !e.isNull() )
+    {
+        QDomElement about = elementDocInfo.namedItem( "about" ).toElement();
+        if ( about.isNull() ) {
+            about = docinfo.createElement( "about" );
+            elementDocInfo.appendChild( about );
+        }
+        QDomElement tmp = KoDom::namedItemNS( e, ooNS::meta, "keyword" );
+        if ( !tmp.isNull() && !tmp.text().isEmpty() )
+        {
+            QDomElement keyword = docinfo.createElement( "keyword" );
+            about.appendChild( keyword );
+            keyword.appendChild( docinfo.createTextNode( tmp.text() ) );
+        }
+    }
+}
+
+KoFilter::ConversionStatus OoUtils::loadAndParse(const QString& fileName, QDomDocument& doc, KoStore *m_store )
+{
+    kdDebug(30518) << "loadAndParse: Trying to open " << fileName << endl;
+
+    if (!m_store->open(fileName))
+    {
+        kdWarning(30519) << "Entry " << fileName << " not found!" << endl;
+        return KoFilter::FileNotFound;
+    }
+    KoFilter::ConversionStatus convertStatus = loadAndParse( m_store->device(),doc, fileName );
+    m_store->close();
+    return convertStatus;
+
+}
+
+KoFilter::ConversionStatus OoUtils::loadAndParse(QIODevice* io, QDomDocument& doc, const QString & fileName)
+{
+    QXmlInputSource source( io );
+    // Copied from QDomDocumentPrivate::setContent, to change the whitespace thing
+    QXmlSimpleReader reader;
+    KoDocument::setupXmlReader( reader, true /*namespaceProcessing*/ );
+
+    // Error variables for QDomDocument::setContent
+    QString errorMsg;
+    int errorLine, errorColumn;
+    if ( !doc.setContent( &source, &reader, &errorMsg, &errorLine, &errorColumn ) )
+    {
+        kdError(30519) << "Parsing error in " << fileName << "! Aborting!" << endl
+            << " In line: " << errorLine << ", column: " << errorColumn << endl
+            << " Error message: " << errorMsg << endl;
+        return KoFilter::ParsingError;
+    }
+
+    kdDebug(30519) << "File " << fileName << " loaded and parsed!" << endl;
+
+    return KoFilter::OK;
+
+}
+
+KoFilter::ConversionStatus OoUtils::loadAndParse(const QString& filename, QDomDocument& doc, KZip * m_zip)
+{
+    kdDebug(30519) << "Trying to open " << filename << endl;
+
+    if (!m_zip)
+    {
+        kdError(30519) << "No ZIP file!" << endl;
+        return KoFilter::CreationError; // Should not happen
+    }
+
+    const KArchiveEntry* entry = m_zip->directory()->entry( filename );
+    if (!entry)
+    {
+        kdWarning(30519) << "Entry " << filename << " not found!" << endl;
+        return KoFilter::FileNotFound;
+    }
+    if (entry->isDirectory())
+    {
+        kdWarning(30519) << "Entry " << filename << " is a directory!" << endl;
+        return KoFilter::WrongFormat;
+    }
+    const KZipFileEntry* f = static_cast<const KZipFileEntry *>(entry);
+    kdDebug(30519) << "Entry " << filename << " has size " << f->size() << endl;
+    QIODevice* io = f->device();
+    KoFilter::ConversionStatus convertStatus = loadAndParse( io,doc, filename );
+    delete io;
+    return convertStatus;
+}
+
+KoFilter::ConversionStatus OoUtils::loadThumbnail( QImage& thumbnail, KZip * m_zip )
+{
+    const QString filename( "Thumbnails/thumbnail.png" );
+    kdDebug(30519) << "Trying to open thumbnail " << filename << endl;
+
+    if (!m_zip)
+    {
+        kdError(30519) << "No ZIP file!" << endl;
+        return KoFilter::CreationError; // Should not happen
+    }
+
+    const KArchiveEntry* entry = m_zip->directory()->entry( filename );
+    if (!entry)
+    {
+        kdWarning(30519) << "Entry " << filename << " not found!" << endl;
+        return KoFilter::FileNotFound;
+    }
+    if (entry->isDirectory())
+    {
+        kdWarning(30519) << "Entry " << filename << " is a directory!" << endl;
+        return KoFilter::WrongFormat;
+    }
+    const KZipFileEntry* f = static_cast<const KZipFileEntry *>(entry);
+    QIODevice* io=f->device();
+    kdDebug(30519) << "Entry " << filename << " has size " << f->size() << endl;
+
+    if ( ! io->open( IO_ReadOnly ) )
+    {
+        kdWarning(30519) << "Thumbnail could not be opened!" <<endl;
+        delete io;
+        return KoFilter::StupidError;
+    }
+
+    QImageIO imageIO( io, "PNG" );
+    if ( ! imageIO.read() )
+    {
+        kdWarning(30519) << "Thumbnail could not be read!" <<endl;
+        delete io;
+        return KoFilter::StupidError;
+    }
+
+    io->close();
+
+    thumbnail = imageIO.image();
+
+    if ( thumbnail.isNull() )
+    {
+        kdWarning(30519) << "Read thumbnail is null!" <<endl;
+        delete io;
+        return KoFilter::StupidError;
+    }
+
+    delete io;
+
+    kdDebug(30519) << "File " << filename << " loaded!" << endl;
+
+    return KoFilter::OK;
+}
